import click
from .gitleaks_sort import sort_command
from .gitleaks_diff import diff_command
from .gitleaks_verify import verify_command
from .gitleaks_merge import merge_command
import logging

@click.group
@click.option('-x', '--debug', 'log_level', default='WARN', help='debug logging level')
def cli(log_level):
    logging.basicConfig(level=logging._nameToLevel.get(log_level))

cli.add_command(sort_command)
cli.add_command(diff_command)
cli.add_command(verify_command)
cli.add_command(merge_command)

if __name__ == "__main__":
    cli()


from email.policy import default
import click
from gitleaks_py.gitleaks_model import GitleaksConfig, AllowList
from . import gitleaks_r as gl_r, gitleaks_w as gl_w, gitleaks_model as model
import logging

logger = logging.getLogger()

def merge(config_files: tuple, title: str=None, dst=None):
    
    allowlist_paths = set()
    allowlist_stopwords = set() 
    rules = []
    duplicate_rules = []
    titles = []

    def _merge_rules(src):
        dst_rule_ids = set([rule.id for rule in rules])
        for rule in src.rules:
            if rule.id in dst_rule_ids:
                duplicate_rules.append(rule.id)
            else:
                rules.append(rule)
                
    def _merge_allowlist(src):
        allowlist = src.allowlist
        if allowlist:
            if allowlist.paths: allowlist_paths.update(allowlist.paths)
            if allowlist.stopwords: allowlist_stopwords.update(allowlist.stopwords)
    
    def merged_title():
        return title if title else ', '.join(titles)
        
    def allowlist():
        paths=list(allowlist_paths) if allowlist_paths else None
        stopwords=list(allowlist_stopwords) if allowlist_stopwords else None
        return AllowList(description="global allowlist", paths=paths, stopwords=stopwords) if allowlist_paths or allowlist_stopwords else None

    # extract
    for file, gl_config in gl_r.load_all(config_files):
        _merge_allowlist(gl_config)
        _merge_rules(gl_config)
        titles.append(gl_config.title)
        
    # transform
    merged_gl_config = GitleaksConfig(merged_title(), allowlist=allowlist(), rules=rules)
            
    # load
    if dst:
        with open(dst, 'wb') as f:
            gl_w.dump(merged_gl_config, f)
    else:
        click.echo(gl_w.dumps(merged_gl_config))

    # warn for dupes
    if duplicate_rules:
        logger.warn(f"Discarded duplicate rules '{duplicate_rules}'")
            
    return merged_gl_config


@click.command('merge')
@click.option('-t', '--title', 'title', help='Title')
@click.option('-d', '--dst', help='Destination file to output to')
@click.argument('config_files', nargs=-1)
def merge_command(config_files: str, title: str=None, dst: str=None):
    merge(config_files, title, dst)

if __name__ == "__main__":
    merge_command()


import click
from . import gitleaks_r as gl_r, gitleaks_w as gl_w, gitleaks_model as model

def sort(config, dst=None):
    
    # extract & transform
    gl = gl_r.load(config)
    
    # load
    if dst:
        with open(dst, 'wb') as f:
            gl_w.dump(gl, f)
    else:
        click.echo(gl_w.dumps(gl))
    return gl
    
@click.command('sort')
@click.option('-d', '--dst', help='Destination file to output to')
@click.argument('config_file', default=model.GITLEAKS_DEFAULT_CONFIG_FILE)
def sort_command(config_file, dst=None):
    sort(config_file, dst)

if __name__ == "__main__":
    sort()



import sys
import click
import os
import tempfile
import shutil
import json
from dataclasses import dataclass, field
from . import gitleaks_r as gl_r
from . import gitleaks_command as gl
import logging

logger = logging.getLogger()

def verify_config(config_files: tuple, secrets_folder='secrets', dst=None):
    
    def _secret_files(id):
        for file in os.listdir(secrets_folder):
            if file.startswith(f'{id}.'):
                yield os.path.join(secrets_folder, file)
    
    def _verify(status, rule_id, secret_file, config_file):
        with CopyTempFile(secret_file, tmpdir) as dst:
            report = gl.detect(config_file, tmpdir)
            rule_violations = report.violations_for_rule_id(rule_id)
            # violations are good as we've detected a secret for the rule
            if not rule_violations:
                status.fail(secret_file)
    
    with tempfile.TemporaryDirectory() as tmpdir:
        failures = []
        for config_file, gl_config in gl_r.load_all(config_files):
            if gl_config.rules:
                for rule in gl_config.rules:
                    status = VerificationStatus(config_file=config_file, rule_id=rule.id)
                    for secret_file in _secret_files(rule.id):
                        status.tested = True
                        _verify(status, rule.id, secret_file, config_file)
                    if not status.valid():
                        failures.append(status)
                
    report = [f.__dict__ for f in failures]
    _json = json.dumps(report, indent=2)
    if dst:
        with open(dst) as fp:
            fp.write(_json)
    else:
        click.echo(_json)
    return report

class CopyTempFile():
    def __init__(self, src: str, tmp: str) -> None:
        self.src = src
        self.tmp = tmp
        self.dst = None
    
    def __enter__(self):
        dst = str(os.path.join(self.tmp, str(os.path.basename(self.src))))
        # only delete file if we put it there
        if (not os.path.exists(dst)):
            self.dst = dst
            shutil.copyfile(self.src, self.dst)
            logger.info(f"Copied '{self.src}' to '{self.dst}'")
        return dst
  
    def __exit__(self, *args):
        if (self.dst):
            os.remove(self.dst)
            logger.info(f"Removed '{self.dst}'")

@dataclass
class VerificationStatus:
    config_file: str
    rule_id: str
    tested: bool = False
    failures: list[str] = field(default_factory=list)
    def fail(self, secret_file):
        self.failures.append(secret_file)
    def valid(self):
        return self.tested and not self.failures

@click.command('verify')
@click.option('-d', '--dst', help='Destination file to output to')
@click.option('-s', '--secrets', default='secrets', help='Folder with secrets to test rules')
@click.argument('config_files', nargs=-1)
def verify_command(config_files: str, secrets: str, dst: str=None):
    report = verify_config(config_files, secrets, dst)
    if report:
        sys.exit(1)
    
if __name__ == "__main__":
    verify_command()



import glob
import tomli
import requests
from requests.structures import CaseInsensitiveDict
from . import gitleaks_model as model

GITLEAKS_DEFAULT_CONFIG_FILE = "https://raw.githubusercontent.com/zricethezav/gitleaks/master/config/gitleaks.toml"

def loadt(__t: dict):
    
    def __rules(__rules):
        if __rules:
            rules = []
            for __r in __rules:
                __cid = CaseInsensitiveDict(__r)
                rules.append(model.Rule(id=__cid.get('id'),
                            description=__cid.get('description'),
                            secretGroup=__cid.get('secretgroup'),
                            regex=__cid.get('regex'),
                            entropy=__cid.get('entropy'),
                            keywords=__cid.get('keywords'),
                            allowlist=__allowlist(__cid.get('allowlist'))))
            return rules
        
    def __allowlist(__al):
        __cid = CaseInsensitiveDict(__al)
        return model.AllowList(description=__cid.get('description'),
                         paths=__cid.get('paths'),
                         stopwords=__cid.get('stopwords')) if __al else None
        
    toml = CaseInsensitiveDict(__t)
    if toml:
        return model.GitleaksConfig(title = toml.get('title'),
                            allowlist=__allowlist(toml.get('allowlist')),
                            rules=__rules(toml.get('rules')))

def loads(__s: str):
    return loadt(tomli.loads(__s))
    
def load(__fp: str):
    return loadt(_load_toml(__fp))
    
def load_all(config_files: tuple):
    for config_file in config_files:
        for file in glob.glob(config_file):
            yield file, load(file)

def _load_toml(__fp):
    if (__fp.startswith('http')):
        with requests.get(__fp) as response:
            body = response.text
            return tomli.loads(body)
    else:
        with open(__fp, mode="rb") as fp:
            return tomli.load(fp)



from dataclasses import dataclass
from typing import List, Set

GITLEAKS_DEFAULT_CONFIG_FILE = "https://raw.githubusercontent.com/zricethezav/gitleaks/master/config/gitleaks.toml"

@dataclass
class AllowList(object):
    description: str==None
    paths: List==None
    stopwords: List==None
    
@dataclass
class Rule(object):
    id: str=None
    description: str=None
    secretGroup: int=None
    regex: str=None
    entropy: float=None
    keywords: List=None
    allowlist: AllowList=None

@dataclass
class GitleaksConfig(object):
    title: str=None
    allowlist: AllowList=None
    rules: List[Rule]=None
    rule_ids: Set[str]=None
    
    def __post_init__(self):
        if self.rules: 
            self.rules.sort(key=lambda r : r.id.lower() if r.id else r.id)
            self.rule_ids = set([rule.id for rule in self.rules])

    def has_rule(self, rule):
        return self.rule_ids and rule.id in self.rule_ids


from dataclasses import dataclass, field
import subprocess
import tempfile
import os.path as os_path
import json

def _read_report(report_file):
    report = GitleaksReport()
    if os_path.exists(report_file):
        with open(report_file) as f:
            _dict = json.load(f)
            for violation in _dict:
                report.violations.append(GitleaksViolation.from_json_dict(violation))
    return report
    
def detect(gl_config_file, scan_directory):
    with tempfile.TemporaryDirectory() as tmpdir:
        report_file = os_path.join(tmpdir, 'report.json')
        args = ['gitleaks', 'detect', '--no-git', '-c', gl_config_file, '-s', scan_directory, '-r', report_file]
        result = subprocess.run(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        return _read_report(report_file)
        
@dataclass
class GitleaksViolation():
    def from_json_dict(__dict):
        return GitleaksViolation(
            __dict.get('Description'),
            __dict.get('StartLine'),
            __dict.get('EndLine'),
            __dict.get('StartColumn'),
            __dict.get('EndColumn'),
            __dict.get('Match'),
            __dict.get('Secret'),
            __dict.get('File'),
            __dict.get('Commit'),
            __dict.get('Entropy'),
            __dict.get('Author'),
            __dict.get('Email'),
            __dict.get('Date'),
            __dict.get('Message'),
            __dict.get('Tags'),
            __dict.get('RuleID'),
            __dict.get('Fingerprint')
        )
        
    description: str=None
    start_line: int=None
    end_line: int=None
    start_column: int=None
    end_column: int=None
    match: str=None
    secret: str=None
    file: str=None
    commit: str=None
    entropy: float=None
    author: str=None
    email: str=None
    date: str=None
    message: str=None
    tags: list[str]=field(default_factory=list)
    rule_id: str=None
    fingerprint: str=None

@dataclass
class GitleaksReport():
    violations: list[GitleaksViolation]=field(default_factory=list)
    def violations_for_rule_id(self, rule_id):
        return [violation for violation in self.violations if violation.rule_id == rule_id]
    


import tomli_w
from . import gitleaks_model as model
from typing import BinaryIO
from dataclasses import asdict

def dumpt(__glc: model.GitleaksConfig):
    
    def __dict_factory(__d):
        return {k: v for (k, v) in __d if v is not None}
        
    def __rules(__rules, toml):
        rules = []
        for __r in __rules or []:
            rules.append(asdict(__r, dict_factory=__dict_factory))
        toml['rules'] = rules
        
    def __allowlist(__al, toml):
        if __al:
            toml['allowlist'] = asdict(__al, dict_factory=__dict_factory)

    toml = {}
    toml['title'] = __glc.title
    __allowlist(__glc.allowlist, toml)
    __rules(__glc.rules, toml)
    return toml

def dumps(__glc: model.GitleaksConfig):
    toml = dumpt(__glc)
    return tomli_w.dumps(toml)

def dump(__glc: model.GitleaksConfig, __fp: BinaryIO):
    toml = dumpt(__glc, )
    tomli_w.dump(toml, __fp)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gitleaks_py']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'requests>=2.28.1,<3.0.0',
 'tomli-w>=1.0.0,<2.0.0',
 'tomli>=2.0.1,<3.0.0']

setup_kwargs = {
    'name': 'gitleaks-py',
    'version': '0.1.1',
    'description': '',
    'long_description': '# Gitleaks configuration utilities\n\n[Gitleaks](https://github.com/zricethezav/gitleaks)  is a SAST tool for detecting and preventing hardcoded secrets like passwords, api keys, and tokens in git repos.\n\n`gitleaks-py` provides a python library and CLI to manage Gitleaks rule configurations:\n\n* Compare configurations using sort and diff\n* Verify rules against fixture files containing secrets\n* Merge rules from multiple files into a single file\n\n## Sort\n\nSort Gitleaks config file by case-insensitive rule ID.\n\n```bash\npython -m gitleaks_py.cli sort [OPTIONS] CONFIG_FILE\n```\n\n* `CONFIG_FILE`\n  File or URL to sort\n\n* `-d`, `--dst`\n  Output destination file. Writes to `std-out` if omitted\n\n## Diff\n\nDiff two config files.\n\n```bash\npython -m gitleaks_py.cli diff [OPTIONS] CONFIG_FILE [DEFAULT_CONFIG_FILE]\n```\n\n* `CONFIG_FILE`\n  File or URL to diff\n\n* `DEFAULT_CONFIG_FILE`\n  File or URL to diff against.\n  Defaults to [gitleaks default config file](https://raw.githubusercontent.com/zricethezav/gitleaks/master/config/gitleaks.toml)\n\n* `-d`, `--dst`\n  Output destination file. Writes to `std-out` if omitted\n\n* `-o`, `--omissions`\n  Display omissions (rules from default config, not found in config)\n\n* `-a`, `--additions`\n  Display additions (rule from config, not found in default config)\n\n## Verify\n\nVerify config file against secrets held in sample files.\n\n```bash\npython -m gitleaks_py.cli verify [OPTIONS] CONFIG_FILE\n```\n\n* `CONFIG_FILE`\n  File or URL to verify\n\n* `-d`, `--dst`\n  Output destination file. Writes to `std-out` if omitted\n\n* `-s`, `--secrets`\n  Folder with secrets to test rules. Defaults to `./secrets`\n\n  Files should be named with the rule id followed by a _dot_.\n  e.g. `uk-gov-notify-api-key.txt`\n\n  Multiple files with secrets can be tested against the same rule by use of an additional suffix.\n  e.g. `uk-gov-notify-api-key.1.txt`, `uk-gov-notify-api-key.2.txt`\n\n## Merge\n\nMerge multiple config files into one\n\n```bash\npython -m gitleaks_py.cli merge [OPTIONS] [CONFIG_FILES]...\n```\n\n* `CONFIG_FILES`\n  A space separated list of files to merge. Glob patterns may be used. e.g. `toml/*.toml`\n\n* `-t`, `--title`\n  Output config title. Joins titles from files if omitted\n\n* `-d`, `--dst`\n  Output destination file. Writes to `std-out` if omitted\n',
    'author': 'Pat',
    'author_email': 'patrick.turner@nhs.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import click
from gitleaks_py.gitleaks_model import GitleaksConfig
from . import gitleaks_r as gl_r, gitleaks_w as gl_w, gitleaks_model as model

def _diff(lhs, rhs):
    additions = []
    allowlist = None
    for rule in lhs.rules:
        if (not rhs.has_rule(rule)):
            additions.append(rule)
    if lhs.allowlist != rhs.allowlist:
        allowlist = lhs.allowlist
    return GitleaksConfig(title=f"Rules in '{lhs.title}' and not in '{rhs.title}'", allowlist=allowlist, rules=additions)

def diff(config_file: str, default_config_file: str, additions: bool=True, dst=None):
    
    # extract
    gl_config=gl_r.load(config_file)
    gl_config.title = config_file
    default_config = gl_r.load(default_config_file)
    default_config.title = default_config_file
    
    # transform
    lhs, rhs = (gl_config, default_config) if additions else (default_config, gl_config)
    diff_config = _diff(lhs, rhs)
    
    # load
    if dst:
        gl_w.dump(diff_config, dst)
    else:
        click.echo(gl_w.dumps(diff_config))
    return diff_config

@click.command('diff')
@click.option('-d', '--dst', help='Destination file to output to')
@click.option('-o', '--omissions', 'additions', flag_value=False, help='Omissions from default config, not found in config')
@click.option('-a', '--additions', 'additions', flag_value=True, default=True, help='Additions from config, not found in default config')
@click.argument('config_file')
@click.argument('default_config_file', default=model.GITLEAKS_DEFAULT_CONFIG_FILE)
def diff_command(config_file: str, default_config_file: str, additions: bool=True, dst=None):
    diff(config_file, default_config_file, additions, dst)

if __name__ == "__main__":
    diff_command()
